#include "Project.h"
#include "io/ArhReader.h"
#include "MeshStructure.h"
#include "Global.h"

bool operator==(DOFVector< double >& l, DOFVector< double >& r) 
{
  if(l.getUsedSize() != r.getUsedSize())
    return false;
  DOFIterator< double > lit(&l, USED_DOFS);
  DOFIterator< double > rit(&r, USED_DOFS);
  lit.reset();
  rit.reset();

  while(!lit.end()) {
    if(abs(*lit - *rit)>1e-12)
      return false;
    ++lit;
    ++rit;
  }
  return true;
}

bool compare(SolutionInformation& info, std::string filename)
{
  if(info.sysVec != NULL) {
    assert(false);
  }else if(info.dofVec != NULL) {
    DOFVector< double > filevec(info.dofVec->getFeSpace(), "fileVec");
    Mesh* solMesh=info.dofVec->getFeSpace()->getMesh();
    std::cout<<"initing swap mesh with name "<<solMesh->getName()<<"\n";
    std::cout<<"swap dimension: "<<solMesh->getDim()<<"\n";
    Mesh fileMesh(solMesh->getName(),solMesh->getDim());
    fileMesh.initialize();
    std::cout<<"reading mesh from file "<<filename<<"\n";
    ArhReader::read(filename, &fileMesh, &filevec);
    DOFVector< double > nsolvec(*(info.dofVec));
    MeshStructure nsolvestructure;
    nsolvestructure.init(solMesh);
    MeshStructure filestructure;
    filestructure.init(&fileMesh);
    return nsolvec == filevec && nsolvestructure.compare(filestructure);
  }else
    assert(false);
  return false;
}
